@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2000-2008 Jens Vagelpohl and Contributors. All Rights Reserved.
+# Copyright (c) 2000-2009 Jens Vagelpohl and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
@@ -17,42 +17,13 @@ $Id: FakeLDAP.py 1485 2008-06-04 16:08:38Z jens $
 
 import base64
 import copy
-import ldap
-import re
-import sha
-
 try:
-    set
-except NameError:
-    from sets import Set as set
-
-# Module-level stuff
-__version__ = '2.fake'
-
-SCOPE_BASE = getattr(ldap, 'SCOPE_BASE')
-SCOPE_ONELEVEL = getattr(ldap, 'SCOPE_ONELEVEL')
-SCOPE_SUBTREE = getattr(ldap, 'SCOPE_SUBTREE')
-
-MOD_ADD = getattr(ldap, 'MOD_ADD')
-MOD_REPLACE = getattr(ldap, 'MOD_REPLACE')
-MOD_DELETE = getattr(ldap, 'MOD_DELETE')
-REFERRAL = getattr(ldap, 'REFERRAL')
-TIMEOUT = getattr(ldap, 'TIMEOUT')
-
-LDAPError = ldap.LDAPError
-SERVER_DOWN = ldap.SERVER_DOWN
-PROTOCOL_ERROR = ldap.PROTOCOL_ERROR
-NO_SUCH_OBJECT = ldap.NO_SUCH_OBJECT
-INVALID_CREDENTIALS = ldap.INVALID_CREDENTIALS
-ALREADY_EXISTS = ldap.ALREADY_EXISTS
-SIZELIMIT_EXCEEDED = ldap.SIZELIMIT_EXCEEDED
-PARTIAL_RESULTS = ldap.PARTIAL_RESULTS
-
-OPT_PROTOCOL_VERSION = None
-OPT_REFERRALS = None
-OPT_NETWORK_TIMEOUT = None
-VERSION2 = None
-VERSION3 = None
+    from hashlib import sha1 as sha_new
+except ImportError:
+    from sha import new as sha_new
+import ldap
+from ldap.dn import explode_dn
+import re
 
 # From http://www.ietf.org/rfc/rfc2254.txt, Section 4
 #
@@ -102,6 +73,22 @@ class Op(object):
         return "Op('%s')" % self.op
 
 class Filter(object):
+    """ A simple representation for search filter elements
+    
+    >>> fltr = Filter('cn', '=', 'joe')
+    >>> repr(fltr)
+    "Filter('cn', '=', 'joe')"
+    >>> fltr == Filter('cn', '=', 'joe')
+    True
+    >>> fltr == Filter('CN', '=', 'joe')
+    True
+    >>> fltr == Filter('cn', '=', 'Joe')
+    False
+    >>> filters = [Filter('CN', '=', 'Zack'), Filter('cn', '=', 'Fred')]
+    >>> filters.sort()
+    >>> filters
+    [Filter('cn', '=', 'Fred'), Filter('CN', '=', 'Zack')]
+    """
 
     def __init__(self, attr, comp, value):
         self.attr = attr
@@ -318,19 +305,14 @@ ANY = parse_query('(objectClass=*)')
 GROUP_OF_UNIQUE_NAMES = parse_query('(objectClass=groupOfUniqueNames)')
 
 
-def initialize(conn_str):
-    """ Initialize a new connection """
-    return FakeLDAPConnection()
-
-def explode_dn(dn, *ign, **ignored):
-    """ Get a DN's elements """
-    return [x.strip() for x in dn.split(',')]
-
 def clearTree():
     TREE.clear()
 
 def addTreeItems(dn):
-    """ Add structure directly to the tree given a DN """
+    """ Add structure directly to the tree given a DN 
+
+    returns the last added tree position for convenience
+    """
     elems = explode_dn(dn)
     elems.reverse()
     tree_pos = TREE
@@ -341,6 +323,8 @@ def addTreeItems(dn):
 
         tree_pos = tree_pos[elem]
 
+    return tree_pos
+
 def apply_filter(tree_pos, base, fltr):
     res = []
     q_key, q_val = fltr.attr, fltr.value
@@ -348,10 +332,14 @@ def apply_filter(tree_pos, base, fltr):
     if q_val.startswith('*') or q_val.endswith('*'):
         if q_val != '*':
             # Wildcard search
-            wildcard = True
-            if q_val.startswith('*'):
+            if q_val.startswith('*') and q_val.endswith('*'):
+                wildcard = 'both'
+                q_val = q_val[1:-1]
+            elif q_val.startswith('*'):
+                wildcard = 'start'
                 q_val = q_val[1:]
-            if q_val.endswith('*'):
+            elif q_val.endswith('*'):
+                wildcard = 'end'
                 q_val = q_val[:-1]
     # Need to find out if tree_pos is a leaf record, it needs different handling
     # Leaf records will appear when doing BASE-scoped searches.
@@ -370,9 +358,18 @@ def apply_filter(tree_pos, base, fltr):
             elif wildcard:
                 found = False
                 for x in val[q_key]:
-                    if x.find(q_val) != -1:
-                        found = True
-                        break
+                    if wildcard == 'start':
+                        if x.endswith(q_val):
+                            found = True
+                            break
+                    elif wildcard == 'end':
+                        if x.startswith(q_val):
+                            found = True
+                            break
+                    else:
+                        if q_val in x:
+                            found = True
+                            break
             elif not q_val in val[q_key]:
                 found = False
             if found:
@@ -381,13 +378,19 @@ def apply_filter(tree_pos, base, fltr):
 
 class FakeLDAPConnection:
 
-    def __init__(self):
-        pass
+    def __init__(self, *args, **kw):
+        self.args = args
+        self.kwargs = kw
+        self.options = {}
+        self._last_bind = None
+        self.start_tls_called = False
 
     def set_option(self, option, value):
-        pass
+        self.options[option] = value
 
     def simple_bind_s(self, binduid, bindpwd):
+        self._last_bind = (self.simple_bind_s, (binduid, bindpwd), {})
+
         if binduid.find('Manager') != -1:
             return 1
 
@@ -395,7 +398,7 @@ class FakeLDAPConnection:
             # Emulate LDAP mis-behavior
             return 1
 
-        sha_obj = sha.new(bindpwd)
+        sha_obj = sha_new(bindpwd)
         sha_dig = sha_obj.digest()
         enc_bindpwd = '{SHA}%s' % base64.encodestring(sha_dig)
         enc_bindpwd = enc_bindpwd.strip()
@@ -407,36 +410,41 @@ class FakeLDAPConnection:
                 break
 
         if not rec_pwd:
-            raise INVALID_CREDENTIALS
+            raise ldap.INVALID_CREDENTIALS
 
         if enc_bindpwd == rec_pwd:
             return 1
         else:
-            raise INVALID_CREDENTIALS
+            raise ldap.INVALID_CREDENTIALS
 
-
-    def search_s(self, base, scope=SCOPE_SUBTREE,
+    def search_s(self, base, scope=ldap.SCOPE_SUBTREE,
                  query='(objectClass=*)', attrs=()):
 
         elems = explode_dn(base)
         elems.reverse()
         tree_pos = TREE
+        tree_pos_dn = ''
 
         for elem in elems:
+            if not tree_pos_dn:
+                tree_pos_dn = elem
+            else:
+                tree_pos_dn = '%s,%s' % (elem, tree_pos_dn)
             if tree_pos.has_key(elem):
                 tree_pos = tree_pos[elem]
+            else:
+                raise ldap.NO_SUCH_OBJECT(elem)
 
         q = parse_query(query)
 
         if cmp_query(q, ANY, strict=True):
             # Return all objects, no matter what class
-            if scope == SCOPE_BASE and tree_pos.get('dn', '') == base:
+            if scope == ldap.SCOPE_BASE and tree_pos_dn == base:
                 # Only if dn matches 'base'
                 return (([base, tree_pos],))
             else:
                 return tree_pos.items()
 
-        res = []
         by_level = {}
         for idx, (operation, filters) in enumerate(explode_query(q)):
             lvl = by_level[idx] = []
@@ -490,7 +498,8 @@ class FakeLDAPConnection:
         if by_level:
             # Return the last one.
             return by_level[idx]
-        return res
+
+        return []
 
 
     def add_s(self, dn, attr_list):
@@ -503,17 +512,20 @@ class FakeLDAPConnection:
         for elem in base:
             if tree_pos.has_key(elem):
                 tree_pos = tree_pos[elem]
+            else:
+                raise ldap.NO_SUCH_OBJECT(elem)
 
         if tree_pos.has_key(rdn):
-            raise ALREADY_EXISTS
-        else:
-            # Add rdn to attributes as well.
-            k, v = rdn.split('=')
-            tree_pos[rdn] = {k:[v]}
-            rec = tree_pos[rdn]
+            raise ldap.ALREADY_EXISTS(rdn)
+
+        # Add rdn to attributes as well.
+        k, v = rdn.split('=')
+        tree_pos[rdn] = {k:[v]}
+        rec = tree_pos[rdn]
+
+        for key, val in attr_list:
+            rec[key] = val
 
-            for key, val in attr_list:
-                rec[key] = val
 
     def delete_s(self, dn):
         elems = explode_dn(dn)
@@ -525,9 +537,13 @@ class FakeLDAPConnection:
         for elem in base:
             if tree_pos.has_key(elem):
                 tree_pos = tree_pos[elem]
+            else:
+                raise ldap.NO_SUCH_OBJECT(elem)
 
-        if tree_pos.has_key(rdn):
-            del tree_pos[rdn]
+        if not tree_pos.has_key(rdn):
+            raise ldap.NO_SUCH_OBJECT(rdn)
+
+        del tree_pos[rdn]
 
     def modify_s(self, dn, mod_list):
         elems = explode_dn(dn)
@@ -539,14 +555,19 @@ class FakeLDAPConnection:
         for elem in base:
             if tree_pos.has_key(elem):
                 tree_pos = tree_pos[elem]
+            else:
+                raise ldap.NO_SUCH_OBJECT(elem)
+
+        if not tree_pos.has_key(rdn):
+            raise ldap.NO_SUCH_OBJECT(rdn)
 
         rec = copy.deepcopy(tree_pos.get(rdn))
 
         for mod in mod_list:
-            if mod[0] == MOD_REPLACE:
+            if mod[0] == ldap.MOD_REPLACE:
                 rec[mod[1]] = mod[2]
-            elif mod[0] == MOD_ADD:
-                cur_val = rec[mod[1]]
+            elif mod[0] == ldap.MOD_ADD:
+                cur_val = rec.get(mod[1], [])
                 cur_val.extend(mod[2])
                 rec[mod[1]] = cur_val
             else:
@@ -573,16 +594,54 @@ class FakeLDAPConnection:
         for elem in base:
             if tree_pos.has_key(elem):
                 tree_pos = tree_pos[elem]
+            else:
+                raise ldap.NO_SUCH_OBJECT(elem)
+
+        if not tree_pos.has_key(rdn):
+            raise ldap.NO_SUCH_OBJECT(rdn)
+
+        if tree_pos.has_key(new_rdn):
+            raise ldap.ALREADY_EXISTS(new_rdn)
 
         rec = tree_pos.get(rdn)
 
         del tree_pos[rdn]
         tree_pos[new_rdn] = rec
 
+    def start_tls_s(self):
+        self.start_tls_called = True
+
+    def result(self, msgid=ldap.RES_ANY, all=1, timeout=-1):
+        return ('partial', [('partial result', {'dn': 'partial result'})])
+
+    def unbind(self):
+        pass
+
+
+class RaisingFakeLDAPConnection(FakeLDAPConnection):
+
+    def setExceptionAndMethod(self, raise_on, exc_class, exc_arg=None):
+        hideaway = '%s_old' % raise_on
+        setattr(self, hideaway, getattr(self, raise_on))
+        def func(*args, **kw):
+            setattr(self, raise_on, getattr(self, hideaway))
+            setattr(self, 'args', args)
+            setattr(self, 'kwargs', kw)
+            if exc_arg:
+                raise exc_class(exc_arg)
+            else:
+                raise exc_class
+        setattr(self, raise_on, func)
+
+
+class FixedResultFakeLDAPConnection(FakeLDAPConnection):
+    search_results = []
+
+    def search_s( self, base, scope=ldap.SCOPE_SUBTREE,
+                  query='(objectClass=*)', attrs=() ):
+        return self.search_results
+
 
 class ldapobject:
     class ReconnectLDAPObject(FakeLDAPConnection):
-        def __init__(self, *ignored):
-            pass
-
-
+        pass
